@@ -1,12 +1,12 @@
 /* wc_lms.h
  *
- * Copyright (C) 2006-2024 wolfSSL Inc.
+ * Copyright (C) 2006-2025 wolfSSL Inc.
  *
  * This file is part of wolfSSL.
  *
  * wolfSSL is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * wolfSSL is distributed in the hope that it will be useful,
@@ -88,6 +88,8 @@
 #ifndef WC_LMS_H
 #define WC_LMS_H
 
+#include <wolfssl/wolfcrypt/types.h>
+
 #if defined(WOLFSSL_HAVE_LMS) && defined(WOLFSSL_WC_LMS)
 
 #include <wolfssl/wolfcrypt/lms.h>
@@ -301,13 +303,13 @@
 #endif
 
 /* Indicates using SHA-256 for hashing. */
-#define LMS_SHA256                  0x00
+#define LMS_SHA256                  0x0000
 /* Indicates using SHA-256/192 for hashing. */
-#define LMS_SHA256_192              0x10
+#define LMS_SHA256_192              0x1000
 /* Mask to get hashing algorithm from type. */
-#define LMS_HASH_MASK               0xf0
+#define LMS_HASH_MASK               0xf000
 /* Mask to get height or Winternitz width from type. */
-#define LMS_H_W_MASK                0x0f
+#define LMS_H_W_MASK                0x0fff
 
 /* LMS Parameters. */
 /* SHA-256 hash, 32-bytes of hash used, tree height of 5. */
@@ -331,24 +333,24 @@
 #define LMOTS_SHA256_N32_W8         0x04
 
 /* SHA-256 hash, 32-bytes of hash used, tree height of 5. */
-#define LMS_SHA256_M24_H5           (0x05 | LMS_SHA256_192)
+#define LMS_SHA256_M24_H5           (0x0a | LMS_SHA256_192)
 /* SHA-256 hash, 32-bytes of hash used, tree height of 10. */
-#define LMS_SHA256_M24_H10          (0x06 | LMS_SHA256_192)
+#define LMS_SHA256_M24_H10          (0x0b | LMS_SHA256_192)
 /* SHA-256 hash, 32-bytes of hash used, tree height of 15. */
-#define LMS_SHA256_M24_H15          (0x07 | LMS_SHA256_192)
+#define LMS_SHA256_M24_H15          (0x0c | LMS_SHA256_192)
 /* SHA-256 hash, 32-bytes of hash used, tree height of 20. */
-#define LMS_SHA256_M24_H20          (0x08 | LMS_SHA256_192)
+#define LMS_SHA256_M24_H20          (0x0d | LMS_SHA256_192)
 /* SHA-256 hash, 32-bytes of hash used, tree height of 25. */
-#define LMS_SHA256_M24_H25          (0x09 | LMS_SHA256_192)
+#define LMS_SHA256_M24_H25          (0x0e | LMS_SHA256_192)
 
 /* SHA-256 hash, 32-bytes of hash used, Winternitz width of 1 bit. */
-#define LMOTS_SHA256_N24_W1         (0x01 | LMS_SHA256_192)
+#define LMOTS_SHA256_N24_W1         (0x05 | LMS_SHA256_192)
 /* SHA-256 hash, 32-bytes of hash used, Winternitz width of 2 bits. */
-#define LMOTS_SHA256_N24_W2         (0x02 | LMS_SHA256_192)
+#define LMOTS_SHA256_N24_W2         (0x06 | LMS_SHA256_192)
 /* SHA-256 hash, 32-bytes of hash used, Winternitz width of 4 bits. */
-#define LMOTS_SHA256_N24_W4         (0x03 | LMS_SHA256_192)
+#define LMOTS_SHA256_N24_W4         (0x07 | LMS_SHA256_192)
 /* SHA-256 hash, 32-bytes of hash used, Winternitz width of 8 bits. */
-#define LMOTS_SHA256_N24_W8         (0x04 | LMS_SHA256_192)
+#define LMOTS_SHA256_N24_W8         (0x08 | LMS_SHA256_192)
 
 typedef struct LmsParams {
     /* Number of tree levels. */
@@ -382,7 +384,11 @@ typedef struct wc_LmsParamsMap {
     /* Identifier of parameters. */
     enum wc_LmsParm id;
     /* String representation of identifier of parameters. */
+#ifdef WOLFSSL_NAMES_STATIC
+    const char str[32]; /* large enough for largest string in wc_lms_map[] */
+#else
     const char* str;
+#endif
     /* LMS parameter set. */
     LmsParams params;
 } wc_LmsParamsMap;
