@@ -1,12 +1,12 @@
 /* io.h
  *
- * Copyright (C) 2006-2024 wolfSSL Inc.
+ * Copyright (C) 2006-2025 wolfSSL Inc.
  *
  * This file is part of wolfSSL.
  *
  * wolfSSL is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * wolfSSL is distributed in the hope that it will be useful,
@@ -57,7 +57,32 @@
     #include "zlib.h"
 #endif
 
-#ifndef USE_WINDOWS_API
+#if defined(__WATCOMC__)
+    #if defined(__NT__)
+    #elif defined(__OS2__)
+        #include <errno.h>
+        #include <os2.h>
+        #include <sys/types.h>
+        #include <os2/types.h>
+        #include <sys/socket.h>
+        #include <arpa/inet.h>
+        #include <netinet/in.h>
+        #include <nerrno.h>
+        #include <sys/ioctl.h>
+
+        typedef int socklen_t;
+    #elif defined(__LINUX__)
+        #include <sys/types.h>
+        #include <errno.h>
+        #include <unistd.h>
+        #include <fcntl.h>
+        #define XFCNTL(fd, flag, block) fcntl((fd), (flag), (block))
+        #include <sys/socket.h>
+        #include <arpa/inet.h>
+        #include <netinet/in.h>
+    #endif
+#elif defined(USE_WINDOWS_API)
+#else
     #if defined(WOLFSSL_LWIP) && !defined(WOLFSSL_APACHE_MYNEWT)
         /* lwIP needs to be configured to use sockets API in this mode */
         /* LWIP_SOCKET 1 in lwip/opt.h or in build */
@@ -204,7 +229,40 @@
 #define SOCKET_RECEIVING 1
 #define SOCKET_SENDING 2
 
-#ifdef USE_WINDOWS_API
+#ifdef __WATCOMC__
+    #if defined(__NT__)
+        /* no epipe yet */
+        #ifndef WSAEPIPE
+            #define WSAEPIPE       -12345
+        #endif
+        #define SOCKET_EWOULDBLOCK WSAEWOULDBLOCK
+        #define SOCKET_EAGAIN      WSAETIMEDOUT
+        #define SOCKET_ETIMEDOUT   WSAETIMEDOUT
+        #define SOCKET_ECONNRESET  WSAECONNRESET
+        #define SOCKET_EINTR       WSAEINTR
+        #define SOCKET_EPIPE       WSAEPIPE
+        #define SOCKET_ECONNREFUSED WSAENOTCONN
+        #define SOCKET_ECONNABORTED WSAECONNABORTED
+    #elif defined(__OS2__)
+        #define SOCKET_EWOULDBLOCK SOCEWOULDBLOCK
+        #define SOCKET_EAGAIN      SOCEAGAIN
+        #define SOCKET_ETIMEDOUT   SOCETIMEDOUT
+        #define SOCKET_ECONNRESET  SOCECONNRESET
+        #define SOCKET_EINTR       SOCEINTR
+        #define SOCKET_EPIPE       SOCEPIPE
+        #define SOCKET_ECONNREFUSED SOCECONNREFUSED
+        #define SOCKET_ECONNABORTED SOCECONNABORTED
+    #elif defined(__UNIX__)
+        #define SOCKET_EWOULDBLOCK EWOULDBLOCK
+        #define SOCKET_EAGAIN      EAGAIN
+        #define SOCKET_ETIMEDOUT   ETIMEDOUT
+        #define SOCKET_ECONNRESET  ECONNRESET
+        #define SOCKET_EINTR       EINTR
+        #define SOCKET_EPIPE       EPIPE
+        #define SOCKET_ECONNREFUSED ECONNREFUSED
+        #define SOCKET_ECONNABORTED ECONNABORTED
+    #endif
+#elif defined(USE_WINDOWS_API)
     /* no epipe yet */
     #ifndef WSAEPIPE
         #define WSAEPIPE       -12345
@@ -416,7 +474,7 @@
         #endif
     #endif
     #ifndef XSOCKOPT_TYPE_OPTVAL_TYPE
-        #ifdef USE_WINDOWS_API
+        #ifndef USE_WINDOWS_API
             #define XSOCKOPT_TYPE_OPTVAL_TYPE void*
         #else
             #define XSOCKOPT_TYPE_OPTVAL_TYPE char*
@@ -520,13 +578,19 @@ WOLFSSL_API  int wolfIO_RecvFrom(SOCKET_T sd, WOLFSSL_BIO_ADDR *addr, char *buf,
 #endif
 #endif /* WOLFSSL_NO_SOCK */
 
+WOLFSSL_API int wolfSSL_BioSend(WOLFSSL* ssl, char *buf, int sz, void *ctx);
+WOLFSSL_API int wolfSSL_BioReceive(WOLFSSL* ssl, char* buf, int sz, void* ctx);
+#ifndef OPENSSL_COEXIST
 /* Preserve API previously exposed */
-WOLFSSL_API int BioSend(WOLFSSL* ssl, char *buf, int sz, void *ctx);
-WOLFSSL_API int BioReceive(WOLFSSL* ssl, char* buf, int sz, void* ctx);
+#define BioSend wolfSSL_BioSend
+#define BioReceive wolfSSL_BioReceive
+#endif
 
 WOLFSSL_LOCAL int SslBioSend(WOLFSSL* ssl, char *buf, int sz, void *ctx);
+#if defined(OPENSSL_EXTRA) || defined(OPENSSL_EXTRA_X509_SMALL)
 WOLFSSL_LOCAL int BioReceiveInternal(WOLFSSL_BIO* biord, WOLFSSL_BIO* biowr,
                                      char* buf, int sz);
+#endif
 WOLFSSL_LOCAL int SslBioReceive(WOLFSSL* ssl, char* buf, int sz, void* ctx);
 #if defined(USE_WOLFSSL_IO)
     /* default IO callbacks */
@@ -605,6 +669,8 @@ WOLFSSL_API void wolfSSL_CTX_SetIORecv(WOLFSSL_CTX *ctx, CallbackIORecv CBIORecv
 WOLFSSL_API void wolfSSL_CTX_SetIOSend(WOLFSSL_CTX *ctx, CallbackIOSend CBIOSend);
 WOLFSSL_API void wolfSSL_SSLSetIORecv(WOLFSSL *ssl, CallbackIORecv CBIORecv);
 WOLFSSL_API void wolfSSL_SSLSetIOSend(WOLFSSL *ssl, CallbackIOSend CBIOSend);
+WOLFSSL_API void wolfSSL_SSLDisableRead(WOLFSSL *ssl);
+WOLFSSL_API void wolfSSL_SSLEnableRead(WOLFSSL *ssl);
 /* deprecated old name */
 #define wolfSSL_SetIORecv wolfSSL_CTX_SetIORecv
 #define wolfSSL_SetIOSend wolfSSL_CTX_SetIOSend
@@ -828,21 +894,36 @@ WOLFSSL_API void wolfSSL_SetIOWriteFlags(WOLFSSL* ssl, int flags);
 
 
 #ifndef XINET_NTOP
-    #define XINET_NTOP(a,b,c,d) inet_ntop((a),(b),(c),(d))
-    #ifdef USE_WINDOWS_API /* Windows-friendly definition */
-        #undef  XINET_NTOP
+    #if defined(__WATCOMC__)
+        #if defined(__OS2__) || defined(__NT__) && \
+                (NTDDI_VERSION >= NTDDI_VISTA)
+            #define XINET_NTOP(a,b,c,d) inet_ntop((a),(b),(c),(d))
+        #else
+            #define XINET_NTOP(a,b,c,d) \
+                strncpy((c),inet_ntoa(*(unsigned *)(b)),(d))
+        #endif
+    #elif defined(USE_WINDOWS_API) /* Windows-friendly definition */
         #define XINET_NTOP(a,b,c,d) InetNtop((a),(b),(c),(d))
+    #else
+        #define XINET_NTOP(a,b,c,d) inet_ntop((a),(b),(c),(d))
     #endif
 #endif
 #ifndef XINET_PTON
-    #define XINET_PTON(a,b,c)   inet_pton((a),(b),(c))
-    #ifdef USE_WINDOWS_API /* Windows-friendly definition */
-        #undef  XINET_PTON
+    #if defined(__WATCOMC__)
+        #if defined(__OS2__) || defined(__NT__) && \
+                (NTDDI_VERSION >= NTDDI_VISTA)
+            #define XINET_PTON(a,b,c)   inet_pton((a),(b),(c))
+        #else
+            #define XINET_PTON(a,b,c)   *(unsigned *)(c) = inet_addr((b))
+        #endif
+    #elif defined(USE_WINDOWS_API) /* Windows-friendly definition */
         #if defined(__MINGW64__) && !defined(UNICODE)
             #define XINET_PTON(a,b,c)   InetPton((a),(b),(c))
         #else
             #define XINET_PTON(a,b,c)   InetPton((a),(PCWSTR)(b),(c))
         #endif
+    #else
+        #define XINET_PTON(a,b,c)   inet_pton((a),(b),(c))
     #endif
 #endif
 
